/*
 * Copyright 2017 Google Inc. All Rights Reserved.
 *
 * This code is confidential and provided to Android partners under NDA. You may modify, compile and
 * distribute the compiled code with your management software to assist with the provisioning and
 * management of Android devices. Other uses require written permission.
 */
package com.google.android.apps.work.dpcsupport;

import android.annotation.TargetApi;
import android.app.admin.DeviceAdminService;
import android.app.admin.DevicePolicyManager;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Build.VERSION;
import android.os.Build.VERSION_CODES;
import androidx.annotation.Nullable;
import androidx.annotation.VisibleForTesting;
import java.util.Set;
import java.util.concurrent.Executor;
import java.util.concurrent.FutureTask;

/** Ensures account management is enabled for only a given set of account types. */
public final class AccountManagementWhitelistEnforcer {

  private final ComponentName adminComponent;
  private final DevicePolicyManagerHelper devicePolicyManagerHelper;
  private final AccountManagementWhitelistHelper helper;
  private final Executor executor;
  private final AuthenticatorTypesGetter authenticatorTypesGetter;

  /**
   * Initializes the account management whitelist enforcer.
   *
   * @param context the application context
   * @param adminComponent the {@link android.app.admin.DeviceAdminReceiver} of the DPC
   */
  public AccountManagementWhitelistEnforcer(Context context, ComponentName adminComponent) {
    this(context, adminComponent, new HandlerThreadExecutor(), new AuthenticatorTypesGetter());
  }

  /**
   * Constructor with the {@link Executor} injected.
   *
   * <p>This is needed because Robolectric enqueues posted {@link Runnable}s and requires the
   * threading internals to be exposed. See discussion <a
   * href="https://github.com/robolectric/robolectric/issues/1993">here</a>.
   */
  @VisibleForTesting
  AccountManagementWhitelistEnforcer(
      Context context,
      ComponentName adminComponent,
      Executor executor,
      AuthenticatorTypesGetter authenticatorTypesGetter) {
    if (context == null) {
      throw new NullPointerException("Context cannot be null");
    }

    if (adminComponent == null) {
      throw new NullPointerException("Admin component cannot be null");
    }
    this.adminComponent = adminComponent;
    this.devicePolicyManagerHelper = new DevicePolicyManagerHelper(context);
    this.helper = new AccountManagementWhitelistHelper(context);
    this.executor = executor;
    this.authenticatorTypesGetter = authenticatorTypesGetter;
  }

  /**
   * Updates the whitelist of application for which account management will be enabled and enforces
   * the whitelist right away. This is safe to call on the main thread.
   *
   * @param whitelist a set of package names for which account management will be enabled, or null
   *     to remove the existing whitelist i.e. enable account management for all packages.
   */
  public void updateAndEnforcePackageWhitelist(@Nullable Set<String> whitelist) {
    if (!devicePolicyManagerHelper.isDeviceOrProfileOwner()) {
      throw new SecurityException("Caller is not a device or profile owner");
    }

    executor.execute(
        helper.getUpdateAndEnforcePackageWhitelistRunnable(
            adminComponent, whitelist, authenticatorTypesGetter));
  }

  /**
   * Updates the whitelist of account types for which account management will be enabled and
   * enforces the whitelist right away. This is safe to call on the main thread.
   *
   * @param whitelist a set of account types for which account management will be enabled, or null
   *     to remove the existing whitelist i.e. enable account management for all account types.
   */
  public void updateAndEnforceAccountManagementWhitelist(@Nullable Set<String> whitelist) {
    if (!devicePolicyManagerHelper.isDeviceOrProfileOwner()) {
      throw new SecurityException("Caller is not a device or profile owner");
    }

    executor.execute(
        helper.getUpdateAndEnforceAuthenticatorWhitelistRunnable(
            adminComponent, whitelist, authenticatorTypesGetter));
  }

  /**
   * Continuously enforces the account management whitelist as new apps are installed. Does nothing
   * if the whitelist has not been set before. To set the whitelist, use {@link
   * AccountManagementWhitelistEnforcer#updateAndEnforceAccountManagementWhitelist(Set)}.
   *
   * <p>This class is a {@link BroadcastReceiver} and must be registered in order to receive the
   * implicit package broadcasts. The registration depends on the target API level:
   *
   * <ul>
   *   <li>Before {@link android.os.Build.VERSION_CODES#O Oreo}, this class must be declared in the
   *       Android Manifest with the <a
   *       href="https://developer.android.com/guide/components/broadcasts.html">&lt;receiver&gt;
   *       element<a>. This is done for you if you have integrated the AndroidManifest.xml of this
   *       library into your own.
   *   <li>For Oreo and above, you must {@link #registerWith(DeviceAdminService) register} this
   *       BroadcastReceiver with a {@link DeviceAdminService}.
   * </ul>
   */
  public static final class ContinuousEnforcer extends BroadcastReceiver {
    private final Executor executor;
    private final AuthenticatorTypesGetter authenticatorTypesGetter;

    /**
     * Indicates whether {@link #registerWith(DeviceAdminService)} has been called during the time
     * this class is loaded. This variable will stay true once set to true. Although not accurate,
     * on Oreo and above this gives a hint whether the {@link ContinuousEnforcer} has been set up.
     */
    static boolean hasRegistered = false;

    public ContinuousEnforcer() {
      this(new HandlerThreadExecutor(), new AuthenticatorTypesGetter());
    }

    @VisibleForTesting
    ContinuousEnforcer(Executor executor, AuthenticatorTypesGetter authenticatorTypesGetter) {
      this.executor = executor;
      this.authenticatorTypesGetter = authenticatorTypesGetter;
    }

    /**
     * Registers a {@link ContinuousEnforcer} with a {@link DeviceAdminService} in order
     * to receive implicit broadcasts on {@link VERSION_CODES#O Oreo} and above.
     *
     * @return the registered ContinuousEnforcer. Be sure to {@link
     *     DeviceAdminService#unregisterReceiver(BroadcastReceiver) unregister} it when the
     *     DeviceAdminService is no longer valid.
     */
    @TargetApi(VERSION_CODES.O)
    public static BroadcastReceiver registerWith(DeviceAdminService deviceAdminService) {
      IntentFilter filter = new IntentFilter(Intent.ACTION_PACKAGE_ADDED);
      filter.addAction(Intent.ACTION_PACKAGE_CHANGED);
      filter.addDataScheme("package");
      ContinuousEnforcer continuousEnforcer = new ContinuousEnforcer();
      deviceAdminService.registerReceiver(continuousEnforcer, filter);
      hasRegistered = true;
      return continuousEnforcer;
    }

    /**
     * {@inheritDoc}
     *
     * <p>Enforces the account management whitelist stored in shared preferences by calling {@link
     * DevicePolicyManager#setAccountManagementDisabled(ComponentName, String, boolean)} on any
     * authenticator account type on the profile or device that is not included in the whitelist.
     */
    @Override
    public void onReceive(Context context, Intent intent) {
      if (VERSION.SDK_INT < VERSION_CODES.LOLLIPOP) {
        return;
      }

      String action = intent.getAction();
      boolean isPackageAddedOrChanged = Intent.ACTION_PACKAGE_ADDED.equals(action)
          || Intent.ACTION_PACKAGE_CHANGED.equals(action);
      if (!isPackageAddedOrChanged || !"package".equals(intent.getScheme())) {
        return;
      }

      final PendingResult pendingResult = goAsync();
      AccountManagementWhitelistHelper helper = new AccountManagementWhitelistHelper(context);

      FutureTask<Void> enforcementTask =
          new FutureTask<Void>(
              helper.getEnforceRunnable(authenticatorTypesGetter), /* result= */ null) {
            @Override
            protected void done() {
              pendingResult.finish();
            }
          };

      executor.execute(enforcementTask);
    }
  }
}
