package com.google.android.apps.work.dpcsupport;

import android.app.admin.DevicePolicyManager;
import android.content.ComponentName;
import android.content.Context;
import android.content.pm.PackageInfo;
import android.os.Build.VERSION_CODES;
import android.util.Log;
import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Removes unwanted applications from managed devices which may have been installed by apps bundled
 * in the system image.
 */
public class DisableSystemInstallers {

  private static final String[] BLACKLISTED_INSTALLERS =
      new String[] {
        "com.altice.android.myapps",
        "com.dti.att",
        "com.dti.cricket",
        "com.dti.millicom",
        "com.dti.blu",
        "com.dti.vizio",
        "com.dti.mts",
        "com.facebook.system",
        "com.sfr.android.sfrjeux",
        "com.LogiaGroup.LogiaDeck"
      };

  private final Context context;
  private final ComponentName admin;
  private final DevicePolicyManager devicePolicyManager;

  public DisableSystemInstallers(Context context, ComponentName admin) {
    this.context = context;
    this.admin = admin;
    this.devicePolicyManager =
        (DevicePolicyManager) context.getSystemService(Context.DEVICE_POLICY_SERVICE);
  }

  /**
   * Called by a device or profile owner to clean up applications which may have been installed by
   * hidden apps in the system image.
   *
   * <p>This can be called in DeviceAdminReceiver.onProfileProvisioningComplete to block these
   * applications from running at all, and can also be called on existing devices to removed apps
   * which have already been installed.
   *
   * @param additionalBlacklistedInstallers additional installers which will be blacklisted along
   *     with the provided blacklist.
   */
  @RequiresApi(VERSION_CODES.LOLLIPOP)
  public void cleanUpBlacklistedInstallers(List<String> additionalBlacklistedInstallers) {
    Set<String> allBlacklistedInstallers = new HashSet<String>();
    allBlacklistedInstallers.addAll(Arrays.asList(BLACKLISTED_INSTALLERS));
    allBlacklistedInstallers.addAll(additionalBlacklistedInstallers);

    // Disable blacklisted installer so they no longer run in the work profile
    for (String installer : allBlacklistedInstallers) {
      Log.i(Constants.TAG, "Removing blacklisted installer " + installer + ".");
      devicePolicyManager.setApplicationHidden(admin, installer, true);
    }

    // Disable any apps installed by blacklisted installers
    List<PackageInfo> installedPackages = context.getPackageManager().getInstalledPackages(0);
    for (PackageInfo installedPackage : installedPackages) {
      @Nullable
      String installerPackage = getInstallerPackageIfPresent(installedPackage.packageName);

      if (installerPackage != null && allBlacklistedInstallers.contains(installerPackage)) {
        Log.i(
            Constants.TAG,
            "Removing " + installedPackage.packageName + " installed by " + installerPackage + ".");
        devicePolicyManager.setApplicationHidden(admin, installedPackage.packageName, true);
      }
    }
  }

  @Nullable
  private String getInstallerPackageIfPresent(String targetPackage) {
    try {
      return context.getPackageManager().getInstallerPackageName(targetPackage);
    } catch (IllegalArgumentException ignored) {
      // getInstallerPackageName can throw IllegalArgumentException if the given package is no
      // longer installed.
    }
    return null;
  }
}
