/*
 * Copyright 2016 Google Inc. All Rights Reserved.
 *
 * This code is confidential and provided to Android partners under NDA. You may modify, compile and
 * distribute the compiled code with your management software to assist with the provisioning and
 * management of Android devices. Other uses require written permission.
 */
package com.google.android.apps.work.dpcsupport;

import android.annotation.TargetApi;
import android.app.admin.DevicePolicyManager;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.content.pm.Signature;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import java.util.Arrays;

/**
 * Handles the message passed from the Play Store app to set the application restrictions.
 *
 * The package name must be provided (using the key {@link #KEY_PACKAGE_NAME}), along with the
 * application restrictions bundle to set (using the key {@link #KEY_APPLICATION_RESTRICTIONS}. To
 * clear the application restrictions, an empty bundle should be passed.
 *
 * Optionally, a boolean can be passed using key {@link #KEY_SEND_BROADCAST} to indicate if an
 * update message broadcast should be sent to alert the app that the managed configurations have
 * changed. This broadcast will use the action {@link #APPLICATION_RESTRICTIONS_CHANGED_ACTION} and
 * will set the flag {@link Intent#FLAG_INCLUDE_STOPPED_PACKAGES}.
 */
@TargetApi(Build.VERSION_CODES.LOLLIPOP)
class ManagedConfigurationsProxyHandler extends Handler {

  public static final String APPLICATION_RESTRICTIONS_CHANGED_ACTION =
      "com.android.vending.APPLICATION_RESTRICTIONS_CHANGED";

  private static final String MANAGED_CONFIGURATIONS_PREFERENCE_FILE =
      "com.google.android.apps.work.dpcsupport.ManagedConfigurationsProxyHandler";
  private static final String MANAGED_CONFIGURATIONS_ADMIN_COMPONENT_KEY =
      "managed_configurations_admin_component";

  public static final String KEY_PACKAGE_NAME = "packageName";
  public static final String KEY_APPLICATION_RESTRICTIONS = "applicationRestrictions";
  public static final String KEY_SEND_BROADCAST = "sendBroadcast";

  private final Context context;

  ManagedConfigurationsProxyHandler(Context context) {
    this.context = context;
  }

  static void enable(Context context, ComponentName admin) {
    saveAdminComponent(context, admin.flattenToString());
    Log.i(Constants.TAG, "Enabled application restrictions proxy.");
  }

  static void disable(Context context) {
    saveAdminComponent(context, null /* adminComponent */);
    Log.i(Constants.TAG, "Disabled application restrictions proxy.");
  }

  @Override
  public void handleMessage(Message msg) {
    ComponentName admin = getAdminComponent();
    if (admin == null) {
      Log.w(Constants.TAG, "Application restrictions proxy is disabled.");
      return;
    }
    if (!isCallerPlayStore(msg.sendingUid)) {
      return;
    }

    String packageName = msg.getData().getString(KEY_PACKAGE_NAME);
    Bundle appRestrictions = msg.getData().getBundle(KEY_APPLICATION_RESTRICTIONS);

    setApplicationRestrictions(admin, packageName, appRestrictions);
    if (msg.getData().getBoolean(KEY_SEND_BROADCAST)) {
      sendApplicationRestrictionsBroadcast(packageName);
    }
  }

  private void setApplicationRestrictions(
      ComponentName admin, String packageName, Bundle appRestrictions) {
    if (packageName == null) {
      throw new IllegalArgumentException("packageName cannot be null.");
    }
    if (appRestrictions == null) {
      throw new IllegalArgumentException("applicationRestrictions bundle cannot be null.");
    }
    DevicePolicyManager devicePolicyManager =
        (DevicePolicyManager) context.getSystemService(Context.DEVICE_POLICY_SERVICE);
    devicePolicyManager.setApplicationRestrictions(admin, packageName, appRestrictions);
    Log.i(Constants.TAG, "Application restrictions set for package " + packageName);
  }

  private void sendApplicationRestrictionsBroadcast(String packageName) {
    Intent intent = new Intent();
    intent.setPackage(packageName);
    intent.setAction(APPLICATION_RESTRICTIONS_CHANGED_ACTION);
    intent.setFlags(Intent.FLAG_INCLUDE_STOPPED_PACKAGES);
    // If the intent is unmatched it will be silently ignored
    context.sendBroadcast(intent);
  }

  /**
   * Checks if the message sent through the bound service was sent by the Play Store app.
   */
  private boolean isCallerPlayStore(int callerUid) {
    PackageManager packageManager = context.getPackageManager();
    String[] callerPackageNames = packageManager.getPackagesForUid(callerUid);
    boolean isCallerPlayStore = false;
    if (callerPackageNames != null) {
      for (String callerPackageName : callerPackageNames) {
        if (callerPackageName.equals(Constants.PLAY_STORE_PACKAGE_NAME)) {
          isCallerPlayStore = true;
        }
      }
    }
    if (!isCallerPlayStore) {
      Log.w(
          Constants.TAG,
          "Proxy caller " + Arrays.toString(callerPackageNames) + " is not Play Store.");
      return false;
    }

    Signature[] signatures;
    try {
      PackageInfo packageInfo =
          packageManager.getPackageInfo(
              Constants.PLAY_STORE_PACKAGE_NAME, PackageManager.GET_SIGNATURES);
      signatures = packageInfo.signatures;
    } catch (NameNotFoundException e) {
      // This should never happen as the package manager gave us the package name.
      throw new IllegalStateException("Play Store signature not found.");
    }

    for (Signature signature : signatures) {
      if (signature.equals(Constants.PLAY_STORE_RELEASE_KEY)
          || signature.equals(Constants.PLAY_STORE_DEBUG_KEY)) {
        return true;
      }
    }
    Log.w(Constants.TAG, "Play Store signature doesn't match.");

    return false;
  }

  private ComponentName getAdminComponent() {
    String adminAsString =
        context
            .getSharedPreferences(MANAGED_CONFIGURATIONS_PREFERENCE_FILE, 0)
            .getString(MANAGED_CONFIGURATIONS_ADMIN_COMPONENT_KEY, null);
    return adminAsString == null ? null : ComponentName.unflattenFromString(adminAsString);
  }

  private static void saveAdminComponent(Context context, String adminComponent) {
    context
        .getSharedPreferences(MANAGED_CONFIGURATIONS_PREFERENCE_FILE, 0)
        .edit()
        .putString(MANAGED_CONFIGURATIONS_ADMIN_COMPONENT_KEY, adminComponent)
        .commit();
  }
}
