/*
 * Copyright 2016 Google Inc. All Rights Reserved.
 *
 * This code is confidential and provided to Android partners under NDA. You may modify, compile and
 * distribute the compiled code with your management software to assist with the provisioning and
 * management of Android devices. Other uses require written permission.
 */
package com.google.android.apps.work.dpcsupport;

import android.annotation.TargetApi;
import android.content.ComponentName;
import android.content.Context;
import android.os.Build;
import android.util.Log;

/**
 * Provides support for Managed Configurations.
 *
 * <p>On Android L and M, a proxy service is enabled, allowing the Play Store to set application
 * restrictions via the DPC.
 *
 * <p>By default the support is disabled, you need to call {@link #enableManagedConfigurations} to
 * enable it. Once the support is enabled it stays enabled until you disable it or the DPC app data
 * is cleared, There is no need to enable it again when the DPC or the device restarts or when the
 * Android OS is updated.
 */
@TargetApi(Build.VERSION_CODES.LOLLIPOP)
public class ManagedConfigurationsSupport {

  private static final String MANAGED_CONFIGURATIONS_PROXY_SERVICE =
      "com.google.android.apps.work.dpcsupport.ManagedConfigurationsProxy";

  private final Context context;
  private final ComponentName adminComponent;

  /**
   * Initiates the support tool for managed configurations.
   * @param context the application context
   * @param adminComponent the {@link android.app.admin.DeviceAdminReceiver} of the DPC
   */
  public ManagedConfigurationsSupport(Context context, ComponentName adminComponent) {
    if (context == null) {
      throw new NullPointerException("Context cannot be null");
    }
    if (adminComponent == null) {
      throw new NullPointerException("Admin component cannot be null");
    }
    this.context = context;
    this.adminComponent = adminComponent;
  }
  /**
   * Enables support for Managed Configurations. The Play Store app will be able to set application
   * restrictions.
   */
  public void enableManagedConfigurations() {
    PreconditionsHelper preconditionsHelper =
        new PreconditionsHelper(context, adminComponent, new DevicePolicyManagerHelper(context));

    if (!preconditionsHelper.ensureHasService(MANAGED_CONFIGURATIONS_PROXY_SERVICE)) {
      Log.e(
          Constants.TAG,
          String.format("Missing service %s in manifest.", MANAGED_CONFIGURATIONS_PROXY_SERVICE));
      throw new IllegalStateException(
          String.format("Missing service %s in manifest.", MANAGED_CONFIGURATIONS_PROXY_SERVICE));
    }

    ManagedConfigurationsProxyHandler.enable(context, adminComponent);
  }

  /**
   * Disables support for Managed Configurations. The Play Store app will not be able to set
   * application restrictions.
   */
  public void disableManagedConfigurations() {
    ManagedConfigurationsProxyHandler.disable(context);
  }
}
