/*
 * Copyright 2016 Google Inc. All Rights Reserved.
 *
 * This code is confidential and provided to Android partners under NDA. You may modify, compile and
 * distribute the compiled code with your management software to assist with the provisioning and
 * management of Android devices. Other uses require written permission.
 */
package com.google.android.apps.work.dpcsupport;

import android.content.Context;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.util.Log;
import androidx.annotation.VisibleForTesting;

/** Utils for checking Play Services and Play Store version. */
class PackageVersionUtils {

  private PackageManager packageManager;

  PackageVersionUtils(Context context) {
    this.packageManager = context.getPackageManager();
  }

  boolean isPlayStoreInstalled() {
    return isPackageInstalled(Constants.PLAY_STORE_PACKAGE_NAME);
  }

  boolean isPlayServicesInstalled() {
    return isPackageInstalled(Constants.PLAY_SERVICES_PACKAGE_NAME);
  }

  boolean isPackageInstalled(String packageName) {
    try {
      packageManager.getPackageInfo(packageName, 0);
      return true;
    } catch (NameNotFoundException e) {
      return false;
    }
  }

  /**
   * Returns the version of {@code packageName} installed on the device.
   *
   * <p>If the app is not installed the returns {@code -1}.
   */
  int getPackageVersion(String packageName) {
    try {
      return packageManager.getPackageInfo(packageName, 0).versionCode;
    } catch (NameNotFoundException e) {
      return -1;
    }
  }

  boolean isPlayStoreUpToDate(int preferredVersion) {
    int installedPlayStoreVersion = getPlayStoreInstalledVersion();

    int preferredPlayStoreVersion =
        Math.max(preferredVersion, PlayServicesAndStoreConstants.PLAY_STORE_MIN_VERSION);
    Log.i(
        Constants.TAG,
        String.format(
            "Play Store preferred version %d, actual version %d.",
            preferredPlayStoreVersion, installedPlayStoreVersion));
    return installedPlayStoreVersion >= preferredPlayStoreVersion;
  }

  boolean isPlayStoreVersionCompatible() {
    int installedPlayStoreVersion = getPlayStoreInstalledVersion();
    Log.i(
        Constants.TAG,
        String.format(
            "Play Store required version %d, actual version %d.",
            PlayServicesAndStoreConstants.PLAY_STORE_MIN_VERSION, installedPlayStoreVersion));
    return installedPlayStoreVersion >= PlayServicesAndStoreConstants.PLAY_STORE_MIN_VERSION;
  }

  /**
   * Returns {@code true} if the Play Services is sufficiently up-to-date to support Android for
   * Work accounts. Otherwise, returns {@code false}.
   */
  boolean isPlayServicesUpToDate(int preferredVersion, int minPlayServicesVersion) {
    int installedPlayServicesVersion = getPlayServicesInstalledVersion();

    int requiredPlayServicesVersion = getRequiredPlayServicesVersion(minPlayServicesVersion);
    int preferredPlayServicesVersion = Math.max(preferredVersion, requiredPlayServicesVersion);
    Log.i(
        Constants.TAG,
        String.format(
            "Play Services preferred version %d, actual version %d.",
            preferredPlayServicesVersion, installedPlayServicesVersion));
    return installedPlayServicesVersion >= preferredPlayServicesVersion;
  }

  boolean isPlayServicesVersionCompatible(int minPlayServicesVersion) {
    int installedPlayServicesVersion = getPlayServicesInstalledVersion();
    int requiredPlayServicesVersion = getRequiredPlayServicesVersion(minPlayServicesVersion);
    Log.i(
        Constants.TAG,
        String.format(
            "Play Services required version %d, actual version %d.",
            requiredPlayServicesVersion, installedPlayServicesVersion));
    return installedPlayServicesVersion >= requiredPlayServicesVersion;
  }

  /**
   * Returns {@code true} if the Chrome is sufficiently up-to-date. Otherwise, returns {@code
   * false}.
   */
  boolean isChromeUpToDate(int requiredChromeVersion) {
    int installedChromeVersion = getChromeInstalledVersion();
    Log.i(
        Constants.TAG,
        String.format(
            "Chrome required version %d, actual version %d.",
            requiredChromeVersion, installedChromeVersion));
    return installedChromeVersion >= requiredChromeVersion;
  }

  private int getPlayStoreInstalledVersion() {
    try {
      return packageManager.getPackageInfo(Constants.PLAY_STORE_PACKAGE_NAME, 0).versionCode;
    } catch (NameNotFoundException e) {
      throw new IllegalArgumentException("Play Store not installed", e);
    }
  }

  private int getPlayServicesInstalledVersion() {
    try {
      return packageManager.getPackageInfo(Constants.PLAY_SERVICES_PACKAGE_NAME, 0).versionCode;
    } catch (NameNotFoundException e) {
      throw new IllegalArgumentException("Play Services not installed", e);
    }
  }

  private int getChromeInstalledVersion() {
    try {
      return packageManager.getPackageInfo(Constants.CHROME_PACKAGE_NAME, 0).versionCode;
    } catch (NameNotFoundException e) {
      throw new IllegalArgumentException("Chrome not installed", e);
    }
  }

  @VisibleForTesting
  static int getRequiredPlayServicesVersion(int minPlayServicesVersion) {
    return Math.max(
        PlayServicesAndStoreConstants.PLAY_SERVICES_MIN_VERSION, minPlayServicesVersion);
  }
}
