package com.google.android.apps.work.dpcsupport;

/**
 * Options for {@link
 * AndroidForWorkAccountSupport#ensureWorkingEnvironment(WorkingEnvironmentOptions,
 * WorkingEnvironmentCallback)}.
 */
public class WorkingEnvironmentOptions {
  /**
   * Constant for specifying there is no preferred version. Used as the argument of {@link
   * Builder#setPreferredPlayServiceVersion(int)} or {@link
   * Builder#setPreferredPlayStoreVersion(int)}.
   */
  public static final int NO_PREFERRED_VERSION = 0;

  /**
   * Preferred version of google play services. Note that the required version will still be
   * enforced if this is lower than that.
   */
  public final int preferredPlayServicesVersion;
  /**
   * Preferred version of google play store. Note that the required version will still be enforced
   * if this is lower than that.
   */
  public final int preferredPlayStoreVersion;

  public final boolean checkPlayServices;
  public final boolean enableWorkAccountAuthenticator;
  public final boolean useAuthenticatedPackageUpdate;

  private WorkingEnvironmentOptions(
      int preferredPlayStoreVersion,
      boolean checkPlayServices,
      int preferredPlayServicesVersion,
      boolean enableWorkAccountAuthenticator,
      boolean useAuthenticatedPackageUpdate) {
    this.preferredPlayStoreVersion = preferredPlayStoreVersion;
    this.checkPlayServices = checkPlayServices;
    this.preferredPlayServicesVersion = preferredPlayServicesVersion;
    this.enableWorkAccountAuthenticator = enableWorkAccountAuthenticator;
    this.useAuthenticatedPackageUpdate = useAuthenticatedPackageUpdate;
  }

  /** Builder class for {@link WorkingEnvironmentOptions} objects. */
  public static class Builder {
    private static final int DEFAULT_PLAY_SERVICES_PREFERRED_VERSION =
        PlayServicesAndStoreConstants.PLAY_SERVICES_PREFERRED_VERSION;

    private static final int DEFAULT_PLAY_STORE_PREFERRED_VERSION =
        PlayServicesAndStoreConstants.PLAY_STORE_PREFERRED_VERSION;

    private int preferredPlayVersion = DEFAULT_PLAY_STORE_PREFERRED_VERSION;

    private boolean checkPlayServices;
    private int preferredPlayServicesVersion = DEFAULT_PLAY_SERVICES_PREFERRED_VERSION;

    private boolean enableWorkAccountAuthenticator;
    private boolean useAuthenticatedPackageUpdate;

    public Builder() {
      this.checkPlayServices = true;
      this.enableWorkAccountAuthenticator = true;
    }

    /**
     * Sets the preferred google play store version. If not set, {@link
     * PlayServicesAndStoreConstants#PLAY_STORE_PREFERRED_VERSION} is the default value. Uses {@link
     * #NO_PREFERRED_VERSION} if there is no preferred version.
     */
    public Builder setPreferredPlayStoreVersion(int preferredPlayVersion) {
      this.preferredPlayVersion = preferredPlayVersion;
      return this;
    }

    public Builder setCheckPlayServices(boolean checkPlayServices) {
      this.checkPlayServices = checkPlayServices;
      return this;
    }

    /**
     * Sets the preferred google play services version. If not set, {@link
     * PlayServicesAndStoreConstants#PLAY_SERVICES_PREFERRED_VERSION} is the default value. Uses
     * {@link #NO_PREFERRED_VERSION} if there is no preferred version.
     */
    public Builder setPreferredPlayServiceVersion(int preferredPlayServicesVersion) {
      this.preferredPlayServicesVersion = preferredPlayServicesVersion;
      return this;
    }

    public Builder setEnableWorkAccountAuthenticator(boolean enableWorkAccountAuthenticator) {
      this.enableWorkAccountAuthenticator = enableWorkAccountAuthenticator;
      return this;
    }

    public Builder setAuthenticatedPackageUpdate(boolean useAuthenticatedPackageUpdate) {
      this.useAuthenticatedPackageUpdate = useAuthenticatedPackageUpdate;
      return this;
    }

    public WorkingEnvironmentOptions build() {
      return new WorkingEnvironmentOptions(
          preferredPlayVersion,
          checkPlayServices,
          preferredPlayServicesVersion,
          enableWorkAccountAuthenticator,
          useAuthenticatedPackageUpdate);
    }
  }
}
